//
// Copyright (C) 2008 Institut fuer Telematik, Universitaet Karlsruhe (TH)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//

#include "AnonPrimitiveBenchmark.h"

AnonPrimitiveBenchmark::AnonPrimitiveBenchmark ()
{
}

AnonPrimitiveBenchmark::~AnonPrimitiveBenchmark ()
{
}

void AnonPrimitiveBenchmark::run ()
{
	ANON_NAME_PAIR		primitive;
	unsigned char*		buffer	= (unsigned char*) malloc (65535);
	unsigned int		buffersizestart; 
	unsigned int		buffersizeend;
	unsigned int		runcount;
	unsigned int		multiplicator = 1;
	string				stat = "";
	time_t				lasttime;
	double				duration;
	char				durationbuf [128];
	string				strbuf;
	char				tmp [128];

	while ((primitive = next ()).second != NULL) {

		cout << "benchmarking " << primitive.first << " ... " << std::endl;
		stat = "";

		//
		// determine the buffer sizes to run
		//
		
		if (	primitive.first.compare ("AnonRandomize")	== 0 ||
				primitive.first.compare ("AnonShuffle")		== 0) {	
			runcount = 50;	
		} else if (primitive.first.compare ("AnonWhitenoise") == 0) {
			runcount = 1;
		} else {
			buffersizestart = 1;
			buffersizeend	= BENCH_MAX_BUFFER_SIZE;
			runcount		= BENCH_RUN_COUNT;
		}

		for (unsigned int i=buffersizestart; i<=buffersizeend; i+=20) {

			multiplicator	= 1;
			lasttime		= time (NULL);

			// run same buffer size multiple times
			for (unsigned int j=0; j<BENCH_RUN_COUNT; j++)  {
				primitive.second->anonimizeBuffer (buffer, i);

				if (j+1 == BENCH_RUN_COUNT && difftime (time (NULL), lasttime) < 1) {
					j = 0;
					multiplicator++;
				}

			} // for (unsigned int j=0; j<BENCH_RUN_COUNT; j++)  

			duration	= 1.0/(double)(multiplicator * BENCH_RUN_COUNT);
			gcvt		(duration, 12, durationbuf); 
			sprintf		(tmp, "%d", i);
			strbuf		= tmp;

			stat += strbuf + "\t" + durationbuf + "\n";

		} // for (unsigned int i=0; i<65536; i++) 

		results.push_back (ANON_RESULT_PAIR (primitive.first, stat));
		
		delete primitive.second;

	} // while ((prim = next ()) != NULL) 		
		
	free (buffer);
}

unsigned int AnonPrimitiveBenchmark::getResultCount ()
{	
	return (unsigned int) results.size ();
}

AnonPrimitiveBenchmark::ANON_RESULT_PAIR AnonPrimitiveBenchmark::getResult (unsigned int idx)
{
	return results [idx];
}

AnonPrimitiveBenchmark::ANON_NAME_PAIR AnonPrimitiveBenchmark::next ()
{
	static int i = 0;

	switch (i++) {
		case  0: return ANON_NAME_PAIR ("AnonHashHmacSha1",			new AnonHashHmacSha1			("key"));	break; 
		case  1: return ANON_NAME_PAIR ("AnonHashSha1",				new AnonHashSha1				());		break; 
		case  2: return ANON_NAME_PAIR ("AnonRandomize",			new AnonRandomize				());		break; 
		case  3: return ANON_NAME_PAIR ("AnonShuffle",				new AnonShuffle					());		break; 
		case  4: return ANON_NAME_PAIR ("AnonWhitenoise",			new AnonWhitenoise				(2));		break;
		
		//case  0: return ANON_NAME_PAIR ("AnonIdentity",				new AnonIdentity				());		break; 
		//case  1: return ANON_NAME_PAIR ("AnonBytewiseHashHmacSha1", new AnonBytewiseHashHmacSha1	("key"));	break; 
		//case  2: return ANON_NAME_PAIR ("AnonBytewiseHashSha1",		new AnonBytewiseHashSha1		());		break; 
		//case  3: return ANON_NAME_PAIR ("AnonConstOverwrite",		new AnonConstOverwrite			(0xFF));	break; 
		//case  4: return ANON_NAME_PAIR ("AnonContinuousChar",		new AnonContinuousChar			());		break; 
		// case  0: return ANON_NAME_PAIR ("AnonCryptoPan",			new AnonCryptoPan				("key"));	break; 
		//case  9: return ANON_NAME_PAIR ("AnonShorten",				new AnonShorten					(0));		break; 
		//case 12: return ANON_NAME_PAIR ("AnonBroadcastHandler",		new AnonBroadcastHandler		());		break;
	}

	return ANON_NAME_PAIR ("", NULL);
}
